{(*}
(*------------------------------------------------------------------------------
 Delphi Code formatter source code 

The Original Code is Settings.pas, released April 2000.
The Initial Developer of the Original Code is Anthony Steele. 
Portions created by Anthony Steele are Copyright (C) 1999-2000 Anthony Steele.
All Rights Reserved. 
Contributor(s): Anthony Steele. 

The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"). you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://www.mozilla.org/NPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied.
See the License for the specific language governing rights and limitations 
under the License.
------------------------------------------------------------------------------*)
{*)}

unit JCFSettings;

interface

uses
    { delphi } Registry,
    { local } SetLog, SetFile,
  SetObfuscate, SetClarify, SetIndent, SetSpaces, SetReturns,
  SetCaps, SetAnyWordCaps,
  SetAlign, SetReplace, SetUses, SetUi;

type

  TSettings = class(TObject)
  private
    fReg: TRegIniFile;

    fcLog: TSetLog;
    fcObfuscate: TSetObfuscate;
    fcClarify: TSetClarify;
    fcReturns: TSetReturns;
    fcSpaces: TSetSpaces;
    fcIndent: TSetIndent;

    fcFile: TSetFile;
    fcCaps: TSetCaps;
    fcSpecificWordCaps: TSetAnyWordCaps;
    fcAlign: TSetAlign;
    fcUses: TSetUses;

    fcReplace: TSetReplace;

    fcUi: TSetUi;

  protected

  public
    constructor Create;
    destructor Destroy; override;

    procedure Read;
    procedure Write;

    property Log: TSetLog read fcLog;
    property FileSettings: TSetFile read FcFile;

    property Obfuscate: TSetObfuscate read fcObfuscate;
    property Clarify: TSetClarify read fcClarify;
    property Indent: TSetIndent read fcIndent;
    property Spaces: TSetSpaces read fcSpaces;
    property Returns: TSetReturns read fcReturns;


    property Caps: TSetCaps read fcCaps;
    property SpecificWordCaps: TSetAnyWordCaps read fcSpecificWordCaps;
    property Align: TSetAlign read fcAlign;
    property Replace: TSetReplace read fcReplace;
    property UsesClause: TSetUses read fcUses;

    // this is not settigns on how to format, it is settings on this program's Ui state
    property UI: TSetUi read fcUi write fcUi;
  end;

const
  ROOT_KEY = '\Software\Jedi\JediCodeFormat';

implementation

uses SysUtils;


  { TSettings }

constructor TSettings.Create;
begin
  inherited;

  // open to HKEY_CURRENT_USER by default
  fReg := TRegIniFile.Create;
  fReg.OpenKey(ROOT_KEY, True);

  fcFile     := TSetFile.Create;
  fcFile.Reg := fReg;

  fcObfuscate     := TSetObfuscate.Create;
  fcObfuscate.Reg := fReg;

  fcClarify     := TSetClarify.Create;
  fcClarify.Reg := fReg;

  fcIndent     := TSetIndent.Create;
  fcIndent.Reg := fReg;

  fcSpaces     := TSetSpaces.Create;
  fcSpaces.Reg := fReg;

  fcReturns := TSetReturns.Create;
  fcReturns.Reg := fReg;

  fcCaps     := TSetCaps.Create;
  fcCaps.Reg := fReg;

  fcSpecificWordCaps     := TSetAnyWordCaps.Create;
  fcSpecificWordCaps.Reg := fReg;

  fcAlign     := TSetAlign.Create;
  fcAlign.Reg := fReg;

  fcReplace     := TSetReplace.Create;
  fcReplace.Reg := fReg;

  fcUses     := TSetUses.Create;
  fcUses.Reg := fReg;

  fcLog     := TSetLog.Create;
  fcLog.Reg := fReg;

  fcUI := TSetUi.Create;
  fcUI.Reg := fReg;

  Read;
end;

destructor TSettings.Destroy;
begin
  Write;

  FreeAndNil(fcLog);
  FreeAndNil(fcFile);
  FreeAndNil(fcObfuscate);
  FreeAndNil(fcClarify);
  FreeAndNil(fcIndent);
  FreeAndNil(fcSpaces);
  FreeAndNil(fcReturns);
  FreeAndNil(fcCaps);
  FreeAndNil(fcSpecificWordCaps);
  FreeAndNil(fcReplace);
  FreeAndNil(fcAlign);
  FreeAndNil(fcUses);

  FreeAndNil(fReg);
  FreeAndNil(fcUI);

  inherited;
end;

procedure TSettings.Read;
begin
  fcLog.Read;
  fcFile.Read;
  fcObfuscate.Read;
  fcClarify.Read;
  fcIndent.Read;
  fcSpaces.Read;
  fcReturns.Read;
  fcCaps.Read;
  fcSpecificWordCaps.Read;
  fcAlign.Read;
  fcReplace.Read;
  fcUses.Read;
  fcUI.Read;
end;

procedure TSettings.Write;
begin
  fcLog.Write;
  fcFile.Write;
  fcObfuscate.Write;
  fcClarify.Write;
  fcIndent.Write;
  fcSpaces.Write;
  fcReturns.Write;
  fcCaps.Write;
  fcSpecificWordCaps.Write;
  fcAlign.Write;
  fcReplace.Write;
  fcUses.Write;
  fcUI.Write;
end;

end.