{(*}
(*------------------------------------------------------------------------------
 Delphi Code formatter source code 

The Original Code is SetIndent.pas, released April 2000.
The Initial Developer of the Original Code is Anthony Steele. 
Portions created by Anthony Steele are Copyright (C) 1999-2000 Anthony Steele.
All Rights Reserved. 
Contributor(s): Anthony Steele. 

The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"). you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://www.mozilla.org/NPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied.
See the License for the specific language governing rights and limitations 
under the License.
------------------------------------------------------------------------------*)
{*)}

unit SetIndent;

interface

uses JCFSetBase, TokenType;

type

  TSetIndent = class(TSetBase)
  private
    fiIndentSpaces: integer;
    fiFirstLevelIndent: integer;
    fbHasFirstLevelIndent: Boolean;

    fbIndentGlobals: boolean;
    fbIndentClasses: boolean;
    fbIndentProcedures: boolean;
    fbIndentBeginEnd: Boolean;

    fiIndentBeginEndSpaces: integer;

    fbKeepCommentsWithCodeInProcs: Boolean;
    fbKeepCommentsWithCodeInGlobals: Boolean;
    fbKeepCommentsWithCodeInClassDef: Boolean;

  public
    constructor Create;

    procedure Read; override;
    procedure Write; override;

    function SpacesForIndentLevel(const piLevel: integer): integer;

    property IndentSpaces: integer read fiIndentSpaces write fiIndentSpaces;
    { first level can be indented differently }
    property FirstLevelIndent: integer read fiFirstLevelIndent write fiFirstLevelIndent;
    property HasFirstLevelIndent: boolean read fbHasFirstLevelIndent write fbHasFirstLevelIndent;

    property IndentGlobals: boolean read fbIndentGlobals write fbIndentGlobals;
    property IndentClasses: boolean read fbIndentClasses write fbIndentClasses;
    property IndentProcedures: boolean read fbIndentProcedures write fbIndentProcedures;

    property IndentBeginEnd: Boolean read fbIndentBeginEnd write fbIndentBeginEnd;
    property IndentBeginEndSpaces: integer read fiIndentBeginEndSpaces write fiIndentBeginEndSpaces;

    property KeepCommentsWithCodeInProcs: Boolean
      read fbKeepCommentsWithCodeInProcs write fbKeepCommentsWithCodeInProcs;
    property KeepCommentsWithCodeInGlobals: Boolean
      read fbKeepCommentsWithCodeInGlobals write fbKeepCommentsWithCodeInGlobals;
    property KeepCommentsWithCodeInClassDef: Boolean
      read fbKeepCommentsWithCodeInClassDef write fbKeepCommentsWithCodeInClassDef;
  end;

implementation

const
  REG_INDENTATION_SPACES = 'IndentationSpaces';

  REG_FIRST_LEVEL_INDENT = 'FirstLevelIndent';
  REG_HAS_FIRST_LEVEL_INDENT = 'HasFirstLevelIndent';

  REG_INDENT_GLOBALS    = 'IndentGlobals';
  REG_INDENT_CLASSES    = 'IndentClasses';
  REG_INDENT_PROCEDURES = 'IndentProcedures';

  REG_INDENT_BEGIN_END = 'IndentBeginEnd';
  REG_INDENT_BEGIN_END_SPACES = 'IndentbeginEndSpaces';

  REG_KEEP_COMMENTS_WTH_CODE_CLASS_DEF = 'KeepCommentsWithCodeInClassDef';
  REG_KEEP_COMMENTS_WTH_CODE_IN_PROCS = 'KeepCommentsWithCodeInProcs';
  REG_KEEP_COMMENTS_WTH_CODE_IN_GLOBALS = 'KeepCommentsWithCodeInGlobals';

constructor TSetIndent.Create;
begin
  inherited;
  SetSection('Indent');
end;

procedure TSetIndent.Read;
begin
  fiIndentSpaces := ReadItem(REG_INDENTATION_SPACES, 2);

  fiFirstLevelIndent := ReadItem(REG_FIRST_LEVEL_INDENT, 2);
  fbHasFirstLevelIndent := ReadItem(REG_HAS_FIRST_LEVEL_INDENT, False);

  fbIndentGlobals    := ReadItem(REG_INDENT_GLOBALS, True);
  fbIndentClasses    := ReadItem(REG_INDENT_CLASSES, True);
  fbIndentProcedures := ReadItem(REG_INDENT_PROCEDURES, True);

  fbIndentBeginEnd  := ReadItem(REG_INDENT_BEGIN_END, False);
  fiIndentBeginEndSpaces := ReadItem(REG_INDENT_BEGIN_END_SPACES, 1);

  fbKeepCommentsWithCodeInGlobals := ReadItem(REG_KEEP_COMMENTS_WTH_CODE_IN_GLOBALS, True);
  fbKeepCommentsWithCodeInProcs := ReadItem(REG_KEEP_COMMENTS_WTH_CODE_IN_PROCS, True);
  fbKeepCommentsWithCodeInClassDef := ReadItem(REG_KEEP_COMMENTS_WTH_CODE_CLASS_DEF, True);
end;

procedure TSetIndent.Write;
begin
  WriteItem(REG_INDENTATION_SPACES, fiIndentSpaces);

  WriteItem(REG_FIRST_LEVEL_INDENT, fiFirstLevelIndent);
  WriteItem(REG_HAS_FIRST_LEVEL_INDENT, fbHasFirstLevelIndent);

  WriteItem(REG_INDENT_GLOBALS, fbIndentGlobals);
  WriteItem(REG_INDENT_CLASSES, fbIndentClasses);
  WriteItem(REG_INDENT_PROCEDURES, fbIndentProcedures);

  WriteItem(REG_KEEP_COMMENTS_WTH_CODE_IN_GLOBALS, fbKeepCommentsWithCodeInGlobals);
  WriteItem(REG_KEEP_COMMENTS_WTH_CODE_IN_PROCS, fbKeepCommentsWithCodeInProcs);
  WriteItem(REG_KEEP_COMMENTS_WTH_CODE_CLASS_DEF, fbKeepCommentsWithCodeInClassDef);

  WriteItem(REG_INDENT_BEGIN_END, fbIndentBeginEnd);
  WriteItem(REG_INDENT_BEGIN_END_SPACES, fiIndentBeginEndSpaces);
end;

function TSetIndent.SpacesForIndentLevel(const piLevel: integer): integer;
begin
  if piLevel <= 0 then
    Result := 0
  else if HasFirstLevelIndent then
    Result := FirstLevelIndent + ((piLevel - 1) * IndentSpaces)
  else
    Result := IndentSpaces * piLevel;
end;


end.