{(*}
(*------------------------------------------------------------------------------
 Delphi Code formatter source code 

The Original Code is SetSpaces.pas, released February 2001.
The Initial Developer of the Original Code is Anthony Steele.
Portions created by Anthony Steele are Copyright (C) 2001 Anthony Steele.
All Rights Reserved.
Contributor(s): Anthony Steele. 

The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"). you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://www.mozilla.org/NPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied.
See the License for the specific language governing rights and limitations 
under the License.
------------------------------------------------------------------------------*)
{*)}

{ mostly spacing and line breaking +options }

unit SetSpaces;

interface

uses JCFSetBase, TokenType;

type

  TSetSpaces = class(TSetBase)
  private
    fbTabsToSpaces: boolean;
    fbSpacesToTabs: boolean;
    fiSpacesForTab: integer;
    fiSpacesPerTab: integer;

    fbFixSpacing: boolean;
    fbSpaceBeforeClassHeritage: boolean;

    fiSpacesBeforeColonVar: integer;
    fiSpacesBeforeColonParam: integer;
    fiSpacesBeforeColonFn: integer;
    fiSpacesBeforeColonClass: integer;
    fiSpacesBeforeColonCaseLabel: integer;
    fiSpacesBeforeColonLabel: integer;

  public
    constructor Create;

    procedure Read; override;
    procedure Write; override;

    property TabsToSpaces: boolean read fbTabsToSpaces write fbTabsToSpaces;
    property SpacesToTabs: boolean read fbSpacesToTabs write fbSpacesToTabs;

    property SpacesPerTab: integer read fiSpacesPerTab write fiSpacesPerTab;
    property SpacesForTab: integer read fiSpacesForTab write fiSpacesForTab;

    property FixSpacing: boolean read fbFixSpacing write fbFixSpacing;

    property SpaceBeforeClassHeritage: boolean read fbSpaceBeforeClassHeritage write
      fbSpaceBeforeClassHeritage;

    property SpacesBeforeColonVar: integer read fiSpacesBeforeColonVar write fiSpacesBeforeColonVar;
    property SpacesBeforeColonParam: integer read fiSpacesBeforeColonParam write fiSpacesBeforeColonParam;
    property SpacesBeforeColonFn: integer read fiSpacesBeforeColonFn write fiSpacesBeforeColonFn;
    property SpacesBeforeColonClass: integer read fiSpacesBeforeColonClass write fiSpacesBeforeColonClass;
    property SpacesBeforeColonCaseLabel: integer read fiSpacesBeforeColonCaseLabel write fiSpacesBeforeColonCaseLabel;
    property SpacesBeforeColonLabel: integer read fiSpacesBeforeColonLabel write fiSpacesBeforeColonLabel;
  end;

implementation

const
  REG_TABS_TO_SPACES = 'TabsToSpaces';
  REG_SPACES_TO_TABS = 'SpacesToTabs';
  REG_SPACES_PER_TAB = 'SpacesPerTab';
  REG_SPACES_FOR_TAB = 'SpacesForTab';

  REG_FIX_SPACING    = 'FixSpacing';

  REG_SPACE_BEFORE_CLASS_HERITAGE = 'SpaceBeforeClassHeritage';
  REG_SPACES_BEFORE_COLON_VAR = 'SpacesBeforeColonVar';
  REG_SPACES_BEFORE_COLON_PARAM = 'SpacesBeforeColonParam';
  REG_SPACES_BEFORE_COLON_FN = 'SpacesBeforeColonFn';
  REG_SPACES_BEFORE_COLON_CLASS = 'SpacesBeforeColonClass';
  REG_SPACES_BEFORE_COLON_CASE_LABEL = 'SpacesBeforeColonCaseLabel';
  REG_SPACES_BEFORE_COLON_LABEL = 'SpacesBeforeColonLabel';

constructor TSetSpaces.Create;
begin
  inherited;
  SetSection('Spaces');
end;

procedure TSetSpaces.Read;
begin
  fbTabsToSpaces  := ReadItem(REG_TABS_TO_SPACES, True);
  fbSpacesToTabs  := ReadItem(REG_SPACES_TO_TABS, False);

  fiSpacesPerTab  := ReadItem(REG_SPACES_PER_TAB, 2);
  fiSpacesForTab  := ReadItem(REG_SPACES_FOR_TAB, 2);

  fbFixSpacing  := ReadItem(REG_FIX_SPACING, True);

  fbSpaceBeforeClassHeritage := ReadItem(REG_SPACE_BEFORE_CLASS_HERITAGE, False);

  fiSpacesBeforeColonVar := ReadItem(REG_SPACES_BEFORE_COLON_VAR, 0);
  fiSpacesBeforeColonParam := ReadItem(REG_SPACES_BEFORE_COLON_PARAM, 0);
  fiSpacesBeforeColonFn := ReadItem(REG_SPACES_BEFORE_COLON_FN, 0);
  fiSpacesBeforeColonClass := ReadItem(REG_SPACES_BEFORE_COLON_CLASS, 0);
  fiSpacesBeforeColonCaseLabel := ReadItem(REG_SPACES_BEFORE_COLON_CASE_LABEL, 0);
  fiSpacesBeforeColonLabel := ReadItem(REG_SPACES_BEFORE_COLON_LABEL, 0);
end;

procedure TSetSpaces.Write;
begin
  WriteItem(REG_TABS_TO_SPACES, fbTabsToSpaces);
  WriteItem(REG_SPACES_TO_TABS, fbSpacesToTabs);
  WriteItem(REG_SPACES_PER_TAB, fiSpacesPerTab);
  WriteItem(REG_SPACES_FOR_TAB, fiSpacesForTab);

  WriteItem(REG_FIX_SPACING, fbFixSpacing);
  WriteItem(REG_SPACE_BEFORE_CLASS_HERITAGE, fbSpaceBeforeClassHeritage);

  WriteItem(REG_SPACES_BEFORE_COLON_VAR, fiSpacesBeforeColonVar);
  WriteItem(REG_SPACES_BEFORE_COLON_PARAM, fiSpacesBeforeColonParam);
  WriteItem(REG_SPACES_BEFORE_COLON_FN, fiSpacesBeforeColonFn);
  WriteItem(REG_SPACES_BEFORE_COLON_CLASS, fiSpacesBeforeColonClass);
  WriteItem(REG_SPACES_BEFORE_COLON_CASE_LABEL, fiSpacesBeforeColonCaseLabel);
  WriteItem(REG_SPACES_BEFORE_COLON_LABEL, fiSpacesBeforeColonLabel);
end;

end.
