{(*}
(*------------------------------------------------------------------------------
 Delphi Code formatter source code 

The Original Code is SetUses.pas, released October 2000.
The Initial Developer of the Original Code is Anthony Steele. 
Portions created by Anthony Steele are Copyright (C) 1999-2000 Anthony Steele.
All Rights Reserved. 
Contributor(s): Anthony Steele. 

The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"). you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://www.mozilla.org/NPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied.
See the License for the specific language governing rights and limitations 
under the License.
------------------------------------------------------------------------------*)
{*)}

unit SetUses;

{ AFS 14 October 2K
 Functonality needed at work, so might as well do it generically
 uses clause find replace can
 - remove a unit name wherever found in a uses clause
 - insert unit names into interface and implementation uses clauses
 - relace unit(s) with a substitute unit wherever they occur
 }

interface

uses
  { delphi } Classes,
  { local } JCFSetBase, TokenType;

type

  TSetUses = class(TSetBase)
  private
    fbRemoveEnabled, fbInsertInterfaceEnabled,
      fbInsertImplementationEnabled, fbFindReplaceEnabled: Boolean;

    fsRemove, fsInsertInterface, fsInsertImplementation, fsFind, fsReplace: TStringList;

  public
    constructor Create;
    destructor Destroy; override;

    procedure Read; override;
    procedure Write; override;

    function GetReplace: string;

    property RemoveEnabled: Boolean read fbRemoveEnabled write fbRemoveEnabled;
    property InsertInterfaceEnabled: Boolean read fbInsertInterfaceEnabled write fbInsertInterfaceEnabled;
    property InsertImplementationEnabled: Boolean read fbInsertImplementationEnabled write fbInsertImplementationEnabled;
    property FindReplaceEnabled: Boolean read fbFindReplaceEnabled write fbFindReplaceEnabled;

    property Remove: TStringList read fsRemove;
    property InsertInterface: TStringList read fsInsertInterface;
    property InsertImplementation: TStringList read fsInsertImplementation;
    property Find: TStringList read fsFind;
    property Replace: TStringList read fsReplace;
  end;

implementation

uses SysUtils;

const
  REG_REMOVE_ENABLED = 'RemoveEnabled';
  REG_INSERT_INTERFACE_ENABLED = 'InsertInterfaceEnabled';
  REG_INSERT_IMPLEMENTATION_ENABLED = 'InsertImplementationEnabled';
  REG_FIND_REPLACE_ENABLED = 'FindReplaceEnabled';

  REG_REMOVE = 'Remove';
  REG_INSERT_INTERFACE = 'InsertInterface';
  REG_INSERT_IMPLEMENTATION = 'InsertImplementation';
  REG_FIND = 'Find';
  REG_REPLACE = 'Replace';

constructor TSetUses.Create;
begin
  inherited;
  SetSection('Uses');

  fsRemove := TStringList.Create;
  fsInsertInterface := TStringList.Create;
  fsInsertImplementation := TStringList.Create;
  fsFind := TStringList.Create;
  fsReplace := TStringList.Create;
end;

destructor TSetUses.Destroy;
begin
  FreeAndNil(fsRemove);
  FreeAndNil(fsInsertInterface);
  FreeAndNil(fsInsertImplementation);
  FreeAndNil(fsFind);
  FreeAndNil(fsReplace);

  inherited;
end;

function TSetUses.GetReplace: string;
var
  liLoop: integer;
begin
  Result := '';

  for liLoop := 0 to fsReplace.Count - 1 do
  begin
    Result := Result + fsReplace[liLoop];
    if liLoop <> (fsReplace.Count - 1) then
      Result := Result + ', ';
  end;

end;

procedure TSetUses.Read;
begin
  fbRemoveEnabled := ReadItem(REG_REMOVE_ENABLED, False);
  fbInsertInterfaceEnabled := ReadItem(REG_INSERT_INTERFACE_ENABLED, False);
  fbInsertImplementationEnabled := ReadItem(REG_INSERT_IMPLEMENTATION_ENABLED, False);
  fbFindReplaceEnabled := ReadItem(REG_FIND_REPLACE_ENABLED, False);

  ReadItem(REG_REMOVE, fsRemove);
  ReadItem(REG_INSERT_INTERFACE, fsInsertInterface);
  ReadItem(REG_INSERT_IMPLEMENTATION, fsInsertImplementation);
  ReadItem(REG_FIND, fsFind);
  ReadItem(REG_REPLACE, fsReplace);
end;

procedure TSetUses.Write;
begin
  WriteItem(REG_REMOVE_ENABLED, fbRemoveEnabled);
  WriteItem(REG_INSERT_INTERFACE_ENABLED, fbInsertInterfaceEnabled);
  WriteItem(REG_INSERT_IMPLEMENTATION_ENABLED, fbInsertImplementationEnabled);
  WriteItem(REG_FIND_REPLACE_ENABLED, fbFindReplaceEnabled);
  
  WriteItem(REG_REMOVE, fsRemove);
  WriteItem(REG_INSERT_INTERFACE, fsInsertInterface);
  WriteItem(REG_INSERT_IMPLEMENTATION, fsInsertImplementation);
  WriteItem(REG_FIND, fsFind);
  WriteItem(REG_REPLACE, fsReplace);
end;

end.

