unit fAllSettings;

{ AFS 7 Sept 2K
  All settings on one form at once
}

interface

uses
  { delphi }
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, ComCtrls, ImgList,
  { local }
  JCFSettings, SettingsFrame;

type
  TFormAllSettings = class(TForm)
    tvFrames: TTreeView;
    pnlSet: TPanel;
    bbOK: TBitBtn;
    bbCancel: TBitBtn;
    procedure tvFramesChange(Sender: TObject; Node: TTreeNode);
    procedure FormCreate(Sender: TObject);
    procedure bbOKClick(Sender: TObject);
    procedure bbCancelClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    frLastFrame: TfrSettingsFrame;
    fSettings: TSettings;

    function GetTreeNodeByName(const psName: string): TTreeNode;

    procedure GetFrameForNode(const pcNode: TTreeNode);
    function GetFrameType(const psName: string): TSettingsFrameClass;
    procedure RemoveAll(const pbSave: Boolean);
  public
    procedure Execute;

    property Settings: TSettings read fSettings write fSettings;
  end;


implementation

{$R *.DFM}


uses frFiles, frObfuscateSettings,
  frClarify, frClarifySpaces, frClarifyIndent,
  frClarifyReturns, frClarifyBlocks, frClarifyAlign,
  frExcludeFiles, frReplace,
  frReservedCapsSettings, frAnyCapsSettings, frUses, frAbout, frbasicSettings;


{ TFormAllSettings }

procedure TFormAllSettings.Execute;
var
  lcNode: TTreeNode;
begin
  { the ide pluggin doesn't have the about box so show it here }

  {$IFDEF IDEPLUGGIN}

    // basic settings page at the start'
    lcNode := tvFrames.Items.GetFirstNode;
    tvFrames.Items.Insert(lcNode, 'Basic');

    //about page at the end
    tvFrames.Items.Add(nil, 'About');
  {$ENDIF}

  tvFrames.FullExpand;

  lcNode := GetTreeNodeByName(Settings.Ui.LastSettingsPage);
  if lcNode <> nil then
    lcNode.Selected := True;

  ShowModal;
end;

procedure TFormAllSettings.GetFrameForNode(const pcNode: TTreeNode);
var
  lcType: TSettingsFrameClass;
  lf: TfrSettingsFrame;
begin
  if pcNode.Data <> nil then
    exit;

  lcType := GetFrameType(pcNode.Text);
  if lcType = nil then
    exit;

  lf := lcType.Create(self);

  { read }
  lf.Settings := Settings;
  lf.Read;

  { show }
  lf.Parent := pnlSet;
  lf.Left := 0;
  lf.Top := 0;
  lf.Width := pnlSet.ClientWidth;
  lf.Height := pnlSet.ClientHeight;

  pcNode.Data := lf;
end;

function TFormAllSettings.GetFrameType(const psName: string): TsettingsFrameClass;
begin
  Result := nil;
  // find the frame?

  // replace this with a record list name -> class map

  if AnsiSameText(psName, 'Logging') then
    Result := TfFiles
  else if AnsiSameText(psName, 'Obfuscate') then
    Result := TfObfuscateSettings
  else if AnsiSameText(psName, 'Clarify') then
    Result := TfClarify
  else if AnsiSameText(psName, 'Spaces') then
    Result := TfClarifySpaces
  else if AnsiSameText(psName, 'Indentation') then
    Result := TfClarifyIndent
  else if AnsiSameText(psName, 'Returns') then
    Result := TfClarifyReturns
  else if AnsiSameText(psName, 'Blocks') then
    Result := TfClarifyBlocks
  else if AnsiSameText(psName, 'Align') then
    Result := TfClarifyAlign
  else if AnsiSameText(psName, 'Object Pascal') then
    Result := TfrReservedCapsSettings
  else if AnsiSameText(psName, 'Any Word') then
    Result := TfrAnyCapsSettings
  else if AnsiSameText(psName, 'Exclusions') then
    Result := TfExcludeFiles
  else if AnsiSameText(psName, 'Find and replace') then
    Result := TfReplace
  else if AnsiSameText(psName, 'Uses') then
    Result := TfUses
  else if AnsiSameText(psName, 'About') then
    Result := TFrameAbout
  else if AnsiSameText(psName, 'Basic') then
    Result := TfrBasic;
end;

procedure TFormAllSettings.tvFramesChange(Sender: TObject; Node: TTreeNode);
var
  lf: TfrSettingsFrame;
begin
  GetFrameForNode(Node);

  if frLastFrame <> nil then
    frLastFrame.Visible := False;

  if Node.Data = nil then
    exit;

  lf := TfrSettingsFrame(Node.Data);
  lf.Visible := True;
  frLastFrame := lf;
end;

procedure TFormAllSettings.FormCreate(Sender: TObject);
begin
  frLastFrame := nil;
end;

procedure TFormAllSettings.bbOKClick(Sender: TObject);
begin
  RemoveAll(True);
  Close;
end;

procedure TFormAllSettings.bbCancelClick(Sender: TObject);
begin
  RemoveAll(False);
  Close;
end;

procedure TFormAllSettings.RemoveAll(const pbSave: Boolean);
var
  liLoop: integer;
  lcItem: TTreeNode;
  lf: TfrSettingsFrame;
begin
  { retrieve frames from the tree nodes and save them }
  for liLoop := 0 to tvFrames.Items.Count - 1 do
  begin
    lcItem := tvFrames.Items[liLoop];
    if lcItem.Data <> nil then
    begin
      lf := lcItem.Data;
      lcItem.Data := nil;

      if pbSave then
        lf.Write;

      lf.Free;
    end;
  end;
end;

function TFormAllSettings.GetTreeNodeByName(const psName: string): TTreeNode;
var
  liLoop: integer;
  lcNode: TTreeNode;
begin
  Result := nil;
  if psName = '' then
    exit;

  for liLoop := 0 to tvFrames.Items.Count - 1 do
  begin
    lcNode := tvFrames.Items[liLoop];
    if AnsiSameText(lcNode.Text, psName) then
    begin
      Result := lcNode;
      break;
    end;
  end;
end;

procedure TFormAllSettings.FormClose(Sender: TObject; var Action: TCloseAction);
begin

  { save the last selected node }
  if tvFrames.Selected <> nil then
    Settings.Ui.LastSettingsPage := tvFrames.Selected.Text;

end;

end.
